@@ -16,7 +16,7 @@ must have a size that is a multiple of nthread  * nfft, where
 nthread is number of threads used, and nfft is the size of the fft. 
 """
 
-__version__ = "0.1.3"
+__version__ = "0.1.4"
 
 from _accelerate_fft_cffi import ffi, lib
 import numpy as np
@@ -268,7 +268,7 @@ def unpack2(a, inplace = False):
     """Unpack vDSP rfft2 data format of shape (..., n, n/2) into regular
     numpy-like rfft2 of shape (..., n, n/2 + 1).
     
-    Optionally, this can be done inplace without calculating the last row.
+    Optionally, this can be done inplace without calculating the last column.
     """
     n0,n1 = a.shape[-2:] #number of rows
     n0_half = n0//2
@@ -319,14 +319,10 @@ def create_fftsetup(logn, double = True):
     #this function must be thread-safe, so we acquire alock here
     with lock:
         if double == True:
-            if fftsetupD.logsize < logn:
-                fftsetupD.create(logn)
-            return fftsetupD.pointer
+            return fftsetupD.create(logn)
         else:
-            if fftsetup.logsize < logn:
-                fftsetup.create(logn)
-            return fftsetup.pointer  
-    
+            return fftsetup.create(logn)
+
 def destroy_fftsetup():
     """Destroys all (double and single precision) setups."""
     with lock:
@@ -343,39 +339,54 @@ class FFTSetupPointer:
     Use :func:`create_fftsetup` to build setups."""
 
     logsize = 0
+    setups = []
     
     def create(self,logsize):
-        self.destroy()
         radix = 2
-        self.logsize = logsize
-        self.pointer = lib.vDSP_create_fftsetup(logsize, radix)
+        if logsize > self.logsize:
+            self.logsize = logsize
+            self.pointer = lib.vDSP_create_fftsetup(logsize, radix)
+            self.setups.append(self.pointer)
+        elif logsize <= 0:
+            raise ValueError("Invalid logsize")
+        return self.pointer
+        
     def destroy(self):
-        try:
-            lib.vDSP_destroy_fftsetup(self.pointer)
+        for pointer in self.setups:
+            lib.vDSP_destroy_fftsetup(pointer)
+        if self.setups != []:
             del self.pointer
-            del self.logsize
-        except AttributeError:
-            pass
+            self.setups.clear()
+        self.logsize = 0
+
+    def __del__(self):
+        self.destroy()
 
 class FFTSetupDPointer:
     """fftsetupD object. This is meant to be initiated only once.
     Use :func:`create_fftsetup` to build setups.""" 
     
     logsize = 0
+    setups = []
     
     def create(self,logsize):
-        self.destroy()
         radix = 2
-        self.logsize = logsize
-        self.pointer = lib.vDSP_create_fftsetupD(logsize, radix)
+        if logsize > self.logsize:
+            self.logsize = logsize
+            self.pointer = lib.vDSP_create_fftsetupD(logsize, radix)
+            self.setups.append(self.pointer)
+        elif logsize <= 0:
+            raise ValueError("Invalid logsize")
+        return self.pointer
+        
     
     def destroy(self):
-        try:
-            lib.vDSP_destroy_fftsetupD(self.pointer)
+        for pointer in self.setups:
+            lib.vDSP_destroy_fftsetupD(pointer)
+        if self.setups != []:
             del self.pointer
-            del self.logsize
-        except AttributeError:
-            pass
+            self.setups.clear()
+        self.logsize = 0
         
     def __del__(self):
         self.destroy()
@@ -475,24 +486,6 @@ def _optimal_flattened_shape(original_shape, dim = 1):
         newshape = (newshape,) + original_shape[-2:]
     return newshape
 
-def _sequential_call(fftfunc,setup, *args,**kwargs):
-    # a simple sequential runner 
-    [fftfunc(setup,*arg,**kwargs) for arg in zip(*args)] 
-
-def _calculate_fft(fftfunc,*args,**kwds):
-    """Runs fft function with given arguments (optionally in parallel 
-    using a ThreadPool)"""
-    nthreads = fft_config["nthreads"]
-
-    if nthreads > 1:
-        pool = Pool(nthreads)
-        workers = [pool.apply_async(_sequential_call, args = (fftfunc,) + arg, kwds = kwds) for arg in zip(*args)] 
-        _ = [w.get() for w in workers]
-        pool.close()
-    else:
-        setup = args[0]
-        args = (setup[0],) + args[1:]
-        _sequential_call(fftfunc,*args, **kwds)
         
       
 def _create_split_complex_pointer(array_real, array_imag, double = False):
@@ -568,11 +561,28 @@ def _init_setup_and_arrays(a, overwrite_x, dim = 1, split_in = False, split_out
     
     return setup, a, out
 
-
 #-----------------
 # Worker functions
 #-----------------
 
+def _sequential_call(fftfunc,setup, *args,**kwargs):
+    # a simple sequential runner 
+    [fftfunc(setup,*arg,**kwargs) for arg in zip(*args)] 
+
+def _calculate_fft(fftfunc,*args,**kwds):
+    """Runs fft function with given arguments (optionally in parallel 
+    using a ThreadPool)"""
+    nthreads = fft_config["nthreads"]
+
+    if nthreads > 1:
+        pool = Pool(nthreads)
+        workers = [pool.apply_async(_sequential_call, args = (fftfunc,) + arg, kwds = kwds) for arg in zip(*args)] 
+        _ = [w.get() for w in workers]
+        pool.close()
+    else:
+        setup = args[0]
+        args = (setup[0],) + args[1:]
+        _sequential_call(fftfunc,*args, **kwds)
 
 def _get_vDSP_fft_inplace(double = False, dim = 1, real_transform = False):
     if double:
